#include <alloca.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libaffin.h"

int
main(int argc, char **argv)
{
	GdkPixbuf *pbuf;
	GdkColorspace cs;
	GError *gerror;
	unsigned char *pix, *pp;
	int w, h, nr_pixels;
	int *r, *g, *b;
	int *rout, *gout, *bout;
	int ok, i, j, rowpix;
	int skip;

	if (argc < 3)
		exit(1);

	g_type_init();

	/* Load image from file. */
	gerror = NULL;
	pbuf = gdk_pixbuf_new_from_file(argv[1], &gerror);

	if (pbuf == NULL) {
		printf("can't open file %s: %s", argv[1], gerror->message);
		exit(1);
	}
        
	/* Obtain height, width and colorspace (currently only RGB). */
	h = gdk_pixbuf_get_height(pbuf);
	w = gdk_pixbuf_get_width(pbuf);
	cs = gdk_pixbuf_get_colorspace(pbuf);

	/* Alpha-channel forms 4 bytes in RGBA form. */
	if (gdk_pixbuf_get_has_alpha(pbuf))
		rowpix = 4;
	else
		rowpix = 3;

	printf("rowpix=%d\n", rowpix);

	nr_pixels = w * h;

	/* Obtain pointer to pixels in the buffer. */
	pix = gdk_pixbuf_get_pixels(pbuf);

	/* PNG and other formats use 4-byte aligned rows. */
	skip = gdk_pixbuf_get_rowstride(pbuf) - w*rowpix;

	printf("%dx%d pixels, rowstride=%d\n", w, h, gdk_pixbuf_get_rowstride(pbuf));

	r = malloc(sizeof(float) * nr_pixels * 3);
	g = r + nr_pixels;
	b = g + nr_pixels;

	rout = malloc(sizeof(float) * nr_pixels * 3);
	gout = rout + nr_pixels;
	bout = gout + nr_pixels;

	pp = pix;

	/* Copy R, G, B channel values into separate arrays. */
	for (i = 0; i < h; i++) {
		for (j = 0; j < w; j++) {
			r[i*w+j] = pp[0];
			g[i*w+j] = pp[1];
			b[i*w+j] = pp[2];
			pp += rowpix;
		}
		/* skip alignment row bytes */
		pp += skip;
	}

	aff_rotate_pic(rout, r, w, h, w/2, h/2, atof(argv[2]));
	aff_rotate_pic(gout, g, w, h, w/2, h/2, atof(argv[2]));
	aff_rotate_pic(bout, b, w, h, w/2, h/2, atof(argv[2]));

	pp = pix;

	/* Write filtered values back to buffer. */
	for (i = 0; i < h; i++) {
		for (j = 0; j < w; j++) {
			pp[0] = rout[i*w+j];
			pp[1] = gout[i*w+j];
			pp[2] = bout[i*w+j];
			pp += rowpix;
		}
		/* skip alignment row bytes */
		pp += skip;
	}

	/* Save image buffer to PNG file. */
	gerror = NULL;
	ok = gdk_pixbuf_save(pbuf, "out.png", "png", &gerror, NULL);

	if (!ok)
		printf("can't save file: %s\n", gerror->message);

	/* This is not necessary since we exit now... */
	free(rout);
	free(r);

	g_object_unref(pbuf);

	return 0;
}

